/******************************************************************************
 *|--------------------------------------------------------------------------|*
 *|                        KoolB Compiler Version 15                         |*
 *|                           Brian C. Becker                                |*
 *|                                                                          |*
 *| Write.h - Handles generating the assembly language, assembling, & linking|*
 *|--------------------------------------------------------------------------|*
 *****************************************************************************/
 
#ifndef Writing_h
#define Writing_h
 
class Writing{
 private:
   string AppData;                //Holds the data section of the program
   string Functions;              //Holds the assembly language for functions
   string FireUpApp;              //Holds initialization routines
   string MainApp;                //Holds the main code for the app
   string FinishUpApp;            //Code to clean up and free resources
   string Resources;              //Raw resources (ie icons, etc)
   string Library;                //Library import functions
   bool   InSubFunction;          //Are we inside a function?
   string SubFunctionFireUpApp;   //If so, this is function intialization area
   string SubFunctionMainApp;     //And the main body of the function
   string SubFunctionFinishUpApp; //Some cleaning up stuff

 public:
   Writing(){
     //Set the default text for each of the sections
     Resources     = ";Resource file - Generated by the KoolB compiler\r\n";
     Library       = ";Our own functions that the KoolB app uses\r\n";
     Library       = ";Library functions to import to the KoolB app\r\n";
     FireUpApp     = "\n\n;Initialize everything to prep the app to run\r\n";
     MainApp       = "\n\n;The main body where the app actually runs\r\n";
     FinishUpApp   = "\n\n;Prepare the app to exit and then terminate\r\n";
     AppData       = "\n\n;Data section of the KoolB app\r\n";
     //Nope, we aren't in a function
     InSubFunction = false;
   }
   void Line(int Section, string Line);
   void File(string FileName);
   void BuildApp(string FileName);
   void EnterSubFunction();
   void ExitSubFunction();

   //Sections of the program
   enum Sections{ToData, ToFireUp, ToMain, ToFinishUp, ToResource,
                 ToLibrary, ToFunction};

};

/******************************************************************************
Line - adds a line to one of the sections
******************************************************************************/
void Writing::Line(int Section, string Asm){
  //Terminate the line with an newline
  string Line = Asm + "\r\n";
  //Then add the line to the correct section
  switch(Section){
    case ToData:
      AppData += Line;
      break;
    case ToFireUp:
      if (InSubFunction == true){
        SubFunctionFireUpApp += Line;
        break;
      }
      FireUpApp += Line;
      break;
    case ToMain:
      if (InSubFunction == true){
        SubFunctionMainApp += Line;
        break;
      }
      MainApp += Line;
      break;
    case ToFinishUp:
      if (InSubFunction == true){
        SubFunctionFinishUpApp += Line;
        break;
      }
      FinishUpApp += Line;
      break;
    case ToResource:
      Resources += Line;
      break;
    case ToLibrary:
      Library += Line;
      break;
    case ToFunction:
      Functions += Line;
      break;
    default:
      //Not a correct section 
      printf("Error: Attempt to write to an invalid section!\r\n");
      exit(1);
      break;
  }
  return;
}

/******************************************************************************
File - write all those sections (which were in memory) to a file
******************************************************************************/
void Writing::File(string FileName){
  string  AsmFileName      = FileName + ".asm";
  string  ResourceFileName = FileName + ".rc";
  ofstream AsmFile;
  ofstream ResourceFile;
  AsmFile.open(AsmFileName.c_str(), ios::out);
  ResourceFile.open(ResourceFileName.c_str(), ios::out);
  AsmFile      << Library     << endl;
  AsmFile      << FireUpApp   << endl;
  AsmFile      << MainApp     << endl;
  AsmFile      << FinishUpApp << endl;
  AsmFile      << Functions   << endl;
  AsmFile      << AppData     << endl;
  ResourceFile << Resources   << endl;
  AsmFile.close();
  ResourceFile.close();
  return;
}

/******************************************************************************
BuildApp - Writes the file to the disk, assembles the file, and links it
******************************************************************************/
void Writing::BuildApp(string FileName){
  //Get only the filename - that way we don't end up with test.bas.asm
  FileName = StripOffExtension(FileName);
  //Write the file to the disk
  File(FileName);
  //If we are running Windows, use NASM (Netwide Assembler) and GoLink
  #ifdef Windows
    Run("Asm\\NASM -E results.txt -f win32 \"" + FileName + ".asm\"");
    printf(" - Assemble time ->  %f seconds\r\n", (double)(clock() - TempTime) / 
                                              (double)CLK_TCK);
    TempTime = clock();
    Run("Asm\\GoRC /r /ni /nw \"" + FileName + ".rc\" > results.txt");
    //Link a console app
    if (AppType == Console){
      Run("Asm\\GoLink /ni /nw /CONSOLE \"" + FileName + ".obj\" \"" + FileName
        + ".res\" kernel32.dll user32.dll gdi32.dll msvcrt.dll > results.txt");
    }
    //Link a GUI app
    if (AppType == GUI){
      Run("Asm\\GoLink /ni /nw \"" + FileName + ".obj\" \"" + FileName +
        ".res\" kernel32.dll user32.dll gdi32.dll msvcrt.dll > results.txt");
    }
    //Link a DLL - exports all the user-defined functions in the database
    if (AppType == DLL){
      Run("Asm\\GoLink /ni /nw /DLL \"" + FileName + ".obj\" /EXPORTS " +
      Data.ListFunctions() + " \"" + FileName + ".res\" kernel32.dll "
      "user32.dll gdi32.dll msvcrt.dll > results.txt");    
    }
    printf(" - Linking time  ->  %f seconds\r\n", (double)(clock() - TempTime) / 
                                              (double)CLOCKS_PER_SEC);
    TempTime = clock();
    //Compress the program or DLL with PEPaCK
    if (Compress == true){
      if (AppType == GUI || AppType == Console){
        Run("Asm\\pepack \"" + FileName + ".exe\" > results.txt");
      }
      if (AppType == DLL){
        Run("Asm\\pepack \"" + FileName + ".dll\" > results.txt");
      }
      printf(" - Compress time ->  %f seconds\r\n", (double)(clock() - TempTime) / 
                                                (double)CLK_TCK);
    }
  #endif
  //If we are using Link, use NASM (Netwide Assembler) and GNU ld
  #ifdef Linux
    Run("nasm -E results.txt -f elf \"" + FileName + ".asm\"");
    printf(" - Assemble time ->  %f seconds\r\n", (double)(clock() - TempTime) / 
                                                (double)CLK_TCK);
    TempTime = clock();
    //GUI and Console are linked identically
    if (AppType == GUI || AppType == Console){
      Run("ld -s -dynamic-linker /lib/ld-linux.* -o \"" + FileName + "\" \"" + 
          FileName + ".o\" /lib/libc.* /lib/libm.* /lib/libdl.* > results.txt");
    }
    //Link a shared library for Linux
    if (AppType == DLL){
      Run("ld -s -dynamic-linker /lib/ld-linux.* -shared -o \"" + FileName + 
          "\" \"" + FileName + ".o\" /lib/libc.* /lib/libm.* /lib/libdl.* "
          "> results.txt");
    }
    printf(" - Linking time  ->  %d seconds\r\n", (double)(clock() - TempTime) / 
                                       (double)CLK_TCK);
    TempTime = clock();
    //To compress, strip the symbols and then compress it with UPX
    if (Compress == true){
      if (AppType == GUI || AppType == Console){
        Run("strip \"" + FileName + "\" > results.txt");
        Run("upx \"" + FileName + "\" > results.txt");
      }
      if (AppType == DLL){
        Run("upx \"" + FileName + ".so\" > results.txt");
      }
      printf(" - Compress time ->  %f seconds\r\n", (double)(clock() - TempTime) / 
                                                  (double)CLK_TCK);
    }
  #endif
  return;
}

/******************************************************************************
EnterSubFunction - reset all the function sections and prepare to build a 
function
******************************************************************************/
void Writing::EnterSubFunction(){
  InSubFunction = true;
  SubFunctionFireUpApp   = "";
  SubFunctionMainApp     = "";
  SubFunctionFinishUpApp = "";
}

/******************************************************************************
ExitSubFunction - combine all the function sections and add them to the 
function section of the app
******************************************************************************/
void Writing::ExitSubFunction(){
  InSubFunction = false;
  Line(ToFunction, SubFunctionFireUpApp);
  Line(ToFunction, SubFunctionMainApp);
  Line(ToFunction, SubFunctionFinishUpApp);
}

#endif
